@@ -345,7 +345,7 @@ our @TRAILING_WHITESPACE_FILES = (
     ['debian/rules'            => qr,[ ]+\n$,], # Allow trailing tabs in Make
 );
 
-sub run {
+sub source {
     my (undef, undef, $info, $proc, $group) = @_;
     my $source_pkg = $proc->pkg_src;
     my $d_files = $info->index_resolved_path('debian/files');
@@ -1025,8 +1025,15 @@ sub _check_html_cruft {
 
     if($blocknumber == 0) {
         if(index($block,'<meta name="generator"') > -1) {
-            if($block =~ m,<meta \s+ name="generator" \s+ content="doxygen,smx)
-            {
+            if(
+                $block =~ m,<meta \s+ name="generator" \s+
+                content="doxygen,smx
+                # Identify and ignore documentation templates by looking
+                # for the use of various interpolated variables.
+                # <http://www.doxygen.nl/manual/config.html#cfg_html_header>
+                && $block
+                !~ m,\$(?:doxygenversion|projectname|projectnumber|projectlogo)\b,
+            ){
                 tag 'source-contains-prebuilt-doxygen-documentation', $entry;
                 return -1;
             }
