@@ -20,11 +20,16 @@ use strict;
 use warnings;
 use autodie;
 
+use IO::Async::Loop;
+use IO::Async::Process;
 use List::MoreUtils qw(first_index none);
 
-use Lintian::Command qw(safe_qx spawn);
 use Lintian::Data;
 use Lintian::Tags qw(tag);
+use Lintian::Util qw(safe_qx);
+
+use constant EMPTY => q{};
+use constant NEWLINE => qq{\n};
 
 # The files that contain error messages from tar, which we'll check and issue
 # tags for if they contain something unexpected, and their corresponding tags.
@@ -38,16 +43,41 @@ our %ERRORS = (
 
 my $EXTRA_MEMBERS = Lintian::Data->new('deb-format/extra-members');
 
-sub run {
+sub always {
     my (undef, $type, $info) = @_;
     my $deb = $info->lab_data_path('deb');
 
     # Run ar t on the *.deb file.  deb will be a symlink to it.
     my $failed; # set to one when something is so bad that we can't continue
-    my $opts = {};
-    my $success = spawn($opts, ['ar', 't', $deb]);
-    if ($success) {
-        my @members = split("\n", ${ $opts->{out} });
+
+    my $loop = IO::Async::Loop->new;
+    my $future = $loop->new_future;
+
+    my @command = ('ar', 't', $deb);
+    my $stdout = EMPTY;
+    my $stderr = EMPTY;
+    my $process = IO::Async::Process->new(
+        command => [@command],
+        stdout => { into => \$stdout },
+        stderr => { into => \$stderr },
+        on_finish => sub {
+            my ($self, $exitcode) = @_;
+            my $status = ($exitcode >> 8);
+
+            if ($status) {
+                $future->fail("Non-zero status $status from @command");
+                return;
+            }
+
+            $future->done("Done with @command");
+            return;
+        });
+
+    $loop->add($process);
+    $loop->await($future);
+
+    if ($future->is_done) {
+        my @members = split(/\n/, $stdout);
         my $count = scalar(@members);
         my ($ctrl_member, $data_member);
         if ($count < 3) {
@@ -170,11 +200,10 @@ sub run {
     } else {
         # unpack will probably fail so we'll never get here, but may as well be
         # complete just in case.
-        my $error = ${ $opts->{err} };
-        $error =~ s/\n.*//s;
-        $error =~ s/^ar:\s*//;
-        $error =~ s/^deb:\s*//;
-        tag 'malformed-deb-archive', "ar error: $error";
+        $stderr =~ s/\n.*//s;
+        $stderr =~ s/^ar:\s*//;
+        $stderr =~ s/^deb:\s*//;
+        tag 'malformed-deb-archive', "ar error: $stderr";
     }
 
     # Check the debian-binary version number.  We probably won't get
