@@ -33,7 +33,7 @@ use Lintian::Check qw(check_spelling spelling_tag_emitter);
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(clean_env do_fork drain_pipe internal_error open_gz);
 
-sub run {
+sub binary {
     my (undef, undef, $info, $proc, $group) = @_;
     my $ginfo = $group->info;
     my (%binary, %link, %manpage, @running_man, @running_lexgrog);
@@ -323,10 +323,11 @@ sub run {
         my $minfo = $manpage{$f};
 
         if ($minfo) {
-            if ($binfo->{sbin} && $minfo->[0]{section} == 1) {
-                tag 'command-in-sbin-has-manpage-in-incorrect-section',
-                  $binfo->{file};
-            }
+            tag 'command-in-sbin-has-manpage-in-incorrect-section',
+              $binfo->{file}
+              if $binfo->{sbin}
+              and $binfo->{file}->is_regular_file
+              and $minfo->[0]{section} == 1;
             if (none { $_->{lang} eq '' } @{$minfo}) {
                 tag 'binary-without-english-manpage', $binfo->{file};
             }
