@@ -33,7 +33,7 @@ use Lintian::Check qw(check_spelling spelling_tag_emitter);
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(internal_error strip);
 
-sub run {
+sub source {
     my (undef, undef, $info, undef, $group) = @_;
 
     # Some (cruft) checks are valid for every patch system, so we need
@@ -292,10 +292,8 @@ sub check_patch {
         my $tag_emitter
           = spelling_tag_emitter('spelling-error-in-patch-description',
             $patch_file);
-        # Check the first line separately to avoid duplicates
-        foreach my $x (split(/\n/, $description, 2)) {
-            check_spelling($x, $group->info->spelling_exceptions,$tag_emitter);
-        }
+        check_spelling($description, $group->info->spelling_exceptions,
+            $tag_emitter, 0);
     }
     # Use --strip=1 to strip off the first layer of directory in case
     # the parent directory in which the patches were generated was
