@@ -27,14 +27,16 @@ use strict;
 use warnings;
 use autodie;
 
-use POSIX qw(strftime);
+use Capture::Tiny qw(capture);
 use List::MoreUtils qw(any);
+use POSIX qw(strftime);
+use Try::Tiny;
 
 use Lintian::Check qw($known_shells_regex);
 use Lintian::Data;
 use Lintian::Relation;
 use Lintian::Tags qw(tag);
-use Lintian::Util qw(do_fork internal_error strip);
+use Lintian::Util qw(internal_error safe_qx strip);
 
 # This is a map of all known interpreters.  The key is the interpreter
 # name (the binary invoked on the #! line).  The value is an anonymous
@@ -89,9 +91,9 @@ my $LEADINSTR
   = '(?:(?:^|[`&;(|{])\s*|(?:if|then|do|while|!)\s+|env(?:\s+[[:alnum:]_]+=(?:\S+|\"[^"]*\"|\'[^\']*\'))*\s+)';
 my $LEADIN = qr/$LEADINSTR/;
 
-# date --date="Sun, 26 Apr 2015 10:28:05 +0800" +%s"
-# <https://lists.debian.org/debian-announce/2015/msg00001.html>
-my $OLDSTABLE_RELEASE = 1_430_015_285;
+# date --date="Sat, 17 Jun 2017 20:22:36 -1000" +%s
+# <https://lists.debian.org/debian-announce/2017/msg00003.html>
+my $OLDSTABLE_RELEASE = 1_497_766_956;
 
 #forbidden command in maintainer scripts
 my $BAD_MAINT_CMD = Lintian::Data->new(
@@ -236,7 +238,7 @@ sub script_tag {
     return;
 }
 
-sub run {
+sub binary {
     my ($pkg, undef, $info) = @_;
 
     my (%executable, %ELF, %scripts, %seen_helper_cmds);
@@ -260,7 +262,11 @@ sub run {
       $info->sorted_index;
 
     my %old_versions;
-    for my $entry ($info->changelog ? $info->changelog->data : ()) {
+    for my $entry (
+        $info->changelog
+        ? @{$info->changelog->entries}
+        : ()
+    ) {
         my $timestamp = $entry->Timestamp // $OLDSTABLE_RELEASE;
         $old_versions{$entry->Version} = $timestamp
           if $timestamp < $OLDSTABLE_RELEASE;
@@ -288,6 +294,12 @@ sub run {
 
         my ($base) = $interpreter =~ m,([^/]*)$,;
 
+        # Ignore Python scripts that are shipped under dist-packages; these
+        # files aren't supposed to be called as scripts.
+        next
+          if $base eq 'python'
+          and $filename =~ m,^usr/lib/python3/dist-packages/,;
+
         # allow exception for .in files that have stuff like #!@PERL@
         next
           if (  $filename =~ m,\.in$,
@@ -1364,16 +1376,8 @@ sub script_is_evil_and_wrong {
 # -n option to check syntax, discarding output and returning the exit status.
 sub check_script_syntax {
     my ($interpreter, $path) = @_;
-    my $fs_path = $path->fs_path;
-    my $pid = do_fork();
-    if ($pid == 0) {
-        open(STDOUT, '>', '/dev/null');
-        open(STDERR, '>&', \*STDOUT);
-        exec $interpreter, '-n', $fs_path
-          or internal_error("cannot exec $interpreter: $!");
-    } else {
-        waitpid $pid, 0;
-    }
+
+    safe_qx($interpreter, '-n', $path->fs_path);
     return $?;
 }
 
