@@ -48,6 +48,8 @@ BEGIN {
     );
 }
 
+use Data::Dumper;
+
 use Capture::Tiny qw(capture_merged);
 use Cwd qw(getcwd);
 use File::Copy;
@@ -57,6 +59,7 @@ use File::Spec::Functions qw(abs2rel rel2abs splitpath splitdir catpath);
 use File::stat;
 use List::Util qw(max);
 use Path::Tiny;
+use Text::Template;
 use Try::Tiny;
 
 use Test::Lintian::ConfigFile qw(read_config write_config);
@@ -110,8 +113,7 @@ sub logged_prepare {
         try {
             # prepare
             prepare($specpath, $runpath, $testset, $force_rebuild);
-        }
-        catch {
+        }catch {
             # catch any error
             $error = $_;
         };
@@ -163,7 +165,7 @@ sub prepare {
 
     # read test data
     my $descpath = "$specpath/$files->{test_specification}";
-    my $testcase = read_config($descpath);
+    my $desc = read_config($descpath);
     $data_epoch= max($data_epoch, stat($descpath)->mtime);
 
     # read test defaults
@@ -171,46 +173,11 @@ sub prepare {
     my $defaults = read_config($descdefaultspath);
     $data_epoch= max($data_epoch, stat($descdefaultspath)->mtime);
 
-    foreach my $key (keys %{$defaults}) {
-        $testcase->{$key} = $defaults->{$key}
-          unless exists $testcase->{$key};
-    }
+    # start with a shallow copy of defaults
+    my $testcase = {%$defaults};
 
-    # record path to specification
-    $testcase->{spec_path} = $specpath;
-
-    unless ($testcase->{testname} && exists $testcase->{version}) {
-        die 'Name or Version missing';
-    }
-
-    $testcase->{source} ||= $testcase->{testname};
-
-    warn "Cannot override Architecture: in test $testcase->{testname}."
-      if length $testcase->{architecture};
-
-    $testcase->{host_architecture} = $ENV{'DEB_HOST_ARCH'}
-      //die 'DEB_HOST_ARCH is not set.';
-
-    $testcase->{standards_version} ||= $ENV{'POLICY_VERSION'}
-      //die 'Could not get POLICY_VERSION.';
-
-    $testcase->{dh_compat_level} //= $ENV{'DEFAULT_DEBHELPER_COMPAT'}
-      //die 'Could not get DEFAULT_DEBHELPER_COMPAT.';
-
-    # add upstream version
-    $testcase->{upstream_version} = $testcase->{version};
-    $testcase->{upstream_version} =~ s/-[^-]+$//;
-    $testcase->{upstream_version} =~ s/(-|^)(\d+):/$1/;
-
-    # version without epoch
-    $testcase->{no_epoch} = $testcase->{version};
-    $testcase->{no_epoch} =~ s/^\d+://;
-
-    unless ($testcase->{prev_version}) {
-        $testcase->{prev_version} = '0.0.1';
-        $testcase->{prev_version} .= '-1'
-          if index($testcase->{version}, '-') > -1;
-    }
+    die "Name missing for $specpath"
+      unless length $desc->{testname};
 
     die 'Outdated test specification (./debian/debian exists).'
       if -e "$specpath/debian/debian";
@@ -245,22 +212,16 @@ sub prepare {
     my @oldrunners = File::Find::Rule->file->name('*.t')->in($runpath);
     unlink(@oldrunners);
 
-    # load skeleton
-    if (exists $testcase->{skeleton}) {
+    my $skeletonname = ($desc->{skeleton} // EMPTY);
+    if (length $skeletonname) {
 
-        # the skeleton we are working with
-        my $skeletonname = $testcase->{skeleton};
+        # load skeleton
         my $skeletonpath = "$testset/skeletons/$skeletonname";
-
         my $skeleton = read_config($skeletonpath);
 
-        foreach my $key (keys %{$skeleton}) {
-            $testcase->{$key} = $skeleton->{$key};
-        }
+        $testcase->{$_} = $skeleton->{$_}for keys %{$skeleton};
     }
 
-    say EMPTY;
-
     # populate working directory with specified template sets
     copy_skeleton_template_sets($testcase->{template_sets},$runpath, $testset)
       if exists $testcase->{template_sets};
@@ -268,40 +229,66 @@ sub prepare {
     # delete templates for which we have originals
     remove_surplus_templates($specpath, $runpath);
 
-    say EMPTY;
-
     # copy test specification to working directory
     my $offset = abs2rel($specpath, $testset);
     say "Copy test specification $offset from $testset to $runpath.";
     copy_dir_contents($specpath, $runpath);
 
-    # get builder name
-    my $buildername = $files->{builder};
-    if (length $buildername) {
-        my $builderpath = "$runpath/$buildername";
+    my $valuefolder = ($testcase->{fill_values_folder} // EMPTY);
+    if (length $valuefolder) {
 
-        # fill builder if needed
-        my $buildertemplate = "$builderpath.in";
-        fill_template($buildertemplate, $builderpath, $testcase, $data_epoch)
-          if -f $buildertemplate;
+        # load all the values in the fill values folder
+        my $valuepath = "$runpath/$valuefolder";
+        my @filepaths
+          = File::Find::Rule->file->name('*.values')->in($valuepath);
 
-        if (-f $builderpath) {
+        for my $filepath (sort @filepaths) {
+            my $values = read_config($filepath);
 
-            # read builder
-            my $builder = read_config($builderpath);
-            die 'Could not read builder data.' unless $builder;
+            $testcase->{$_} = $values->{$_}for keys %{$values};
+        }
+    }
 
-            # adjust age threshold
-            $data_epoch= max($data_epoch, stat($builderpath)->mtime);
+    # add individual settings after skeleton
+    $testcase->{$_} = $desc->{$_}for keys %{$desc};
 
-            # transfer builder data to test case, but do not override
-            foreach my $key (keys %{$builder}) {
-                $testcase->{$key} = $builder->{$key}
-                  unless exists $testcase->{$key};
-            }
+    # record path to specification
+    $testcase->{spec_path} = $specpath;
 
-            # delete builder
-            unlink($builderpath);
+    # add other helpful info to testcase
+    $testcase->{source} ||= $testcase->{testname};
+
+    # record our effective data age as date, unless given
+    $testcase->{date} ||= rfc822date($data_epoch);
+
+    warn "Cannot override Architecture: in test $testcase->{testname}."
+      if length $testcase->{architecture};
+
+    $testcase->{host_architecture} = $ENV{'DEB_HOST_ARCH'}
+      //die 'DEB_HOST_ARCH is not set.';
+
+    $testcase->{standards_version} ||= $ENV{'POLICY_VERSION'}
+      //die 'Could not get POLICY_VERSION.';
+
+    $testcase->{dh_compat_level} //= $ENV{'DEFAULT_DEBHELPER_COMPAT'}
+      //die 'Could not get DEFAULT_DEBHELPER_COMPAT.';
+
+    # add additional version components
+    if (length $testcase->{version}) {
+
+        # add upstream version
+        $testcase->{upstream_version} = $testcase->{version};
+        $testcase->{upstream_version} =~ s/-[^-]+$//;
+        $testcase->{upstream_version} =~ s/(-|^)(\d+):/$1/;
+
+        # version without epoch
+        $testcase->{no_epoch} = $testcase->{version};
+        $testcase->{no_epoch} =~ s/^\d+://;
+
+        unless ($testcase->{prev_version}) {
+            $testcase->{prev_version} = '0.0.1';
+            $testcase->{prev_version} .= '-1'
+              unless $testcase->{type} eq 'native';
         }
     }
 
@@ -317,8 +304,11 @@ sub prepare {
     combine_fields($testcase, 'build_conflicts', COMMA . SPACE,
         'default_build_conflicts', 'extra_build_conflicts');
 
-    # record our effective data age as date, unless given
-    $testcase->{date} ||= rfc822date($data_epoch);
+    # fill testcase with itself; do it twice to make sure all is done
+    $testcase = fill_hash_from_hash($testcase);
+    $testcase = fill_hash_from_hash($testcase);
+
+    say EMPTY;
 
     # fill remaining templates
     fill_skeleton_templates($testcase->{fill_targets},
@@ -378,8 +368,38 @@ sub combine_fields {
     return;
 }
 
+sub fill_hash_from_hash {
+    my ($hashref, $delimiters) = @_;
+
+    my %origin = %{$hashref};
+    my %destination;
+
+    # fill hash with itself
+    for my $key (keys %origin) {
+
+        my $template = $origin{$key} // EMPTY;
+        my $filler= Text::Template->new(TYPE => 'STRING', SOURCE => $template);
+        croak("Cannot read template $template: $Text::Template::ERROR")
+          unless $filler;
+
+        my $generated
+          = $filler->fill_in(HASH => \%origin, DELIMITERS => $delimiters);
+        croak("Could not create string from template $template")
+          unless defined $generated;
+        $destination{$key} = $generated;
+    }
+
+    return \%destination;
+}
+
 =back
 
 =cut
 
 1;
+
+# Local Variables:
+# indent-tabs-mode: nil
+# cperl-indent-level: 4
+# End:
+# vim: syntax=perl sw=4 sts=4 sr et
